<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Base class for all actions.
 * You must derive your action's classes from this class.<br><br>
 * 
 * In the constructor you <b>must</b> define one or more function to validate
 * the input.<br>
 * Use the method setCheckFunction() to do that.<br><br>
 * 
 * Example to validate input:<br>
 * Every field (with the name of the form!!) must have an associative array with<br>
 * the check type and his parameters.<br><br>
 *
 * $definitions is an associative array with the description of the checks.<br>
 * $names is an associative array with the name of the fields, used for the error message.<br>
 * $errorMsgs is an associative array with the possible errors. For every check of the field,
 *            it should contain a description of the error. {field} is a placeholder for the name
 *            of the field (see $names). {checkValue} is a placeholder for the given value for the
 *            check (for example: 'min' => 2).<br>
 *            If $errorMsg does not contain a message for the current error, it will just return
 *            '{field} is not valid' (parsed with GetText)<br>
 *
 * $definition has to be created as follow:
 * 
 * <code>
 * $definition['field on form'] = array('type' => myMVC_BaseAction::$VALIDATE_xxx, params);
 * </code>
 * 
 * In myMVC_BaseAction there are many $VALIDATE_xxx defines to validate the different input
 * fields.<br>
 * Every validate-check can have more parameters.<br>
 * In the following example-code you can find the validate-checks with their parameters
 * explained.<br>
 * You can give more than one parameter for every check.<br>
 * 
 * Valid parameters for <b>all</b> checks (but $VALIDATE_EXTERN and $VALIDATE_UPLOAD) are:<br>
 * 
 * <b>empty</b>         true or false. It decides wheter an empty field should be accepted.<br>
 * <b>range</b>         array of valid values for the checked field.<br>
 * <b>rangei</b>        same as <b>range</b>, but case insensitive (just for $VALIDATE_STRING,
 *                      $VALIDATE_EMAIL and $VALIDATE_WEB).<br>
 * <b>rangesql</b>      SQL-query to get the valid values for the checked field.<br>
 * <b>rangesqli</b>     same as <b>rangesql</b>, but case insensitive (just for $VALIDATE_STRING,
 *                      $VALIDATE_EMAIL and $VALIDATE_WEB).<br>
 * <b>comparedb</b>     Execute an SQL-query and compare the result with the given value.<br>
 *                      In the query you can use {checkValue} as placeholder for the user's given
 *                      input value. See example to use it.<br>
 *                      WARNING: the query <b>must</b> return just <b>one</b> record with <b>one</b>
 *                      column, otherwise it throws an exception.<br>
 * <b>comparefield</b>  like comparedb, but it compares two input fields. Useful for action like
 *                      "password change", when the user has to give the password twice.
 * 
 * $VALIDATE_NUMBER     has 'min' and 'max', too, to restrict the valid numbers into a range.<br>
 * $VALIDATE_DATETIME   has 'format' to give the format of the date (see strftime). This
 *                      parameter is mandatory!<br>
 * $VALIDATE_STRING     has the parameters 'minlen' and 'maxlen' to restrict the length of
 *                      the input string, and the parameter 'regexp' to check the string against
 *                      a given regular expression.<br>
 * $VALIDATE_EXTERN     has <b>just</b> the parameter 'checkFunction'. See the example.
 * $VALIDATE_UPLOAD     has the parameters 'minlen' (in KB), 'maxlen' (in KB), 'empty' and
 *                      'allowed_types'. See the example. PLEASE REMENBER TO SET 'upload_max_filesize'
 *                      in your php.ini!!
 * 
 * If a required parameter wasn't given, the Framework generates an Exception.
 *
 * <code>
 *   public function __construct($request, $session)
 *   {
 *     parent::__construct($request, $session);
 *     $this->setCheckFunction('POST', 'myValidateFunction');
 *   }
 * 
 *   public function myValidateFunction()
 *   {
 *     $definitions = array(
 *         'testBoolean' => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN),
 *         'testBooleanNull' => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN,
 *                                              'empty' => true),
 *         'testNumber' => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER),
 *         'testNumberNull' => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER,
 *                                              'empty' => true),
 *         'testNumberRange1' => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER,
 *                                              'min' => 2, 'max' => 10),
 *         'testNumberRange2' => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER,
 *                                              'range' => array(1, 3, 5, 7, 9)),
 *         'testNumberRangeSQL' => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER,
 *                                              'rangesql' => 'SELECT id FROM test'),
 *         'testDate' => array('type' => myMVC_BaseAction::$VALIDATE_DATETIME,
 *                                              'format' => _('%m/%d/%Y')),
 *         'testDateNull' => array('type' => myMVC_BaseAction::$VALIDATE_DATETIME,
 *                                              'format' => _('%m/%d/%Y'), 'empty' => true),
 *                                            // see strftime for format
 *         'testDateTime' => array('type' => myMVC_BaseAction::$VALIDATE_DATETIME,
 *                                              'format' => _('%m/%d/%Y %H:%M:%S')),
 *         'testDateTimeNull' => array('type' => myMVC_BaseAction::$VALIDATE_DATETIME,
 *                                              'format' => _('%m/%d/%Y %H:%M:%S'), 'empty' => true),
 *         'testEMail' => array('type' => myMVC_BaseAction::$VALIDATE_EMAIL),
 *         'testEMailNull' => array('type' => myMVC_BaseAction::$VALIDATE_EMAIL,
 *                                              'empty' => true),
 *         'testEMailRange' => array('type' => myMVC_BaseAction::$VALIDATE_EMAIL,
 *                                              'range' => array('test@test.de', 'test@test.com')),
 *         'testEMailRangeSQL' => array('type' => myMVC_BaseAction::$VALIDATE_EMAIL,
 *                                              'rangesql' => 'SELECT email FROM user WHERE active = 1'),
 *         'testWeb' => array('type' => myMVC_BaseAction::$VALIDATE_WEB),
 *         'testWebNull' => array('type' => myMVC_BaseAction::$VALIDATE_WEB, 'empty' => true),
 *         'testWebRange' => array('type' => myMVC_BaseAction::$VALIDATE_WEB,
 *                                              'range' => array('www.test.de', 'www.test.com')),
 *         'testWebRangeSQL' => array('type' => myMVC_BaseAction::$VALIDATE_WEB,
 *                                              'rangesql' => 'SELECT name FROM domain WHERE active = 1'),
 *         'testDomain' => array('type' => myMVC_BaseAction::$VALIDATE_WEB, 'justdomain' => true),
 *         'testString1' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20),
 *         'testString2' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20,
 *                                              'minlen' => 5),
 *         'testString3' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20,
 *                                              'empty' => false),
 *         'testStringRange' => array('type' => myMVC_BaseAction::$VALIDATE_STRING,
 *                                              'range' => array('A', 'B', 'C')),
 *         'testStringRangeSQL' => array('type' => myMVC_BaseAction::$VALIDATE_STRING,
 *                                              'rangesql' => 'SELECT name FROM test'),
 *         'testStringRegExp' => array('type' => myMVC_BaseAction::$VALIDATE_STRING,
 *                                              'regexp' => '/^x.*=3$/'),
 *         'testExtraFunction' => array('type' => myMVC_BaseAction::$VALIDATE_EXTERN,
 *                                              'checkFunction' => 'MyClass::checkValue'),
 *         'testNewLogin' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20, 'minlen' => 5,
 *                                              'empty' => false,
 *                                              'comparedb' => array('query' => "SELECT COUNT(*) FROM user WHERE login = {checkValue}",
 *                                                                   'compareType' => '==', 'compareValue' => 0)),
 *         'testPass1' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20, 'minlen' => 5,
 *                                              'empty' => true),
 *         'testPass2' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20, 'minlen' => 5,
 *                                              'empty' => true,
 *                                              'comparefield' => array('field' => 'testPass1', 'compareType' => '==')),
 *         'testFile' => array('type' => myMVC_BaseAction::$VALIDATE_UPLOAD,
 *                                              'minlen' => 1024, 'maxlen' => 65535, 'allowed_types' = array('text/plain', 'text/html'),
 *                                              'empty' => false),
 *     );
 *     $names = array(
 *         'field1' => _('Field ONE'),
 *         'field2' => _('Field TWO')
 *     );
 *     $errorMsgs = array(
 *         'field1' => array(myMVC_BaseAction::$ERROR_EMPTY => sprintf(_("%s cannot be empty'"),
 *                                                              '{field}'),
 *                           myMVC_BaseAction::$ERROR_MIN   => sprintf(_("%s must be at least %s char long'"),
 *                                                              '{field}', '{checkValue}'),
 *     );
 *     return parent::validateInput($definitions, $names, $errorMsgs);
 *   }
 * </code>
 * It is possible to use an external function to validate input:
 * 
 * <code>
 *   public static function checkValue($request, $value, &$errMsg)
 *   {
 *      if($value != 'abcdef')
 *     {
 *       $errMsg = sprintf(_("Invalid value: %s", $value));
 *       return false;
 *     }
 *     else
 *       return true;
 *   }
 * </code>
 * 
 * this function must return true if data are valid, and false otherwise. In this case,
 * $errMsg can contain an error message, that will be parsed with $errorMsgs (placeholder: {externError}).
 * 
 * If you want to call the Action as AJAX, you can (but not must) define the methods
 * performAjax() and manageInvalidDataAjax() with same parameters of perform() and manageInvalidData().
 * To validate the data, you should pass "AJAX" as first parameter to setCheckFunction().
 * So you can have an Action that supports normal calls (GET and/or POST) and AJAX
 */
abstract class myMVC_BaseAction extends myMVC_BasePage
{
  protected $renderEngine;
  protected $request;
  protected $session;
  private $checkFunctionAll, $checkFunctionPost, $checkFunctionGet, $checkFunctionAjax;

  /**
   * Check for a boolean value
   */
  public static $VALIDATE_BOOLEAN             = 'bool';
  /**
   * Check for a numeric value
   */
  public static $VALIDATE_NUMBER              = 'number';
  /**
   * Check for a date/time value
   */
  public static $VALIDATE_DATETIME            = 'dateTime';
  /**
   * Check for an E-Mail
   */
  public static $VALIDATE_EMAIL               = 'email';
  /**
   * Check for an URL
   */
  public static $VALIDATE_WEB                 = 'web';
  /**
   * Check for a generic string
   */
  public static $VALIDATE_STRING              = 'string';
  /**
   * Check using external function
   */
  public static $VALIDATE_EXTERN              = 'extern';
  /**
   * Check for upload file
   */
  public static $VALIDATE_UPLOAD              = 'upload';

  /**
   * Error not boolean
   */
  public static $ERROR_NOTBOOL                = 'notbool';
  /**
   * Error not date/time
   */
  public static $ERROR_NOTDATETIME            = 'notdatetime';
  /**
   * Error not E-Mail
   */
  public static $ERROR_NOTEMAIL               = 'notemail';
  /**
   * Error not numeric
   */
  public static $ERROR_NOTNUMBER              = 'notnumber';
  /**
   * Error not string
   */
  public static $ERROR_NOTSTRING              = 'notstring';
  /**
   * Error not URL
   */
  public static $ERROR_NOTWEB                 = 'notweb';
  /**
   * Error greater than given maximum
   */
  public static $ERROR_MAX                    = 'max';
  /**
   * Error lesser than given minimum
   */
  public static $ERROR_MIN                    = 'min';
  /**
   * Error longer than given maximum
   */
  public static $ERROR_MAXLEN                 = 'maxlen';
  /**
   * Error smaller than given minimum
   */
  public static $ERROR_MINLEN                 = 'minlen';
  /**
   * Error empty
   */
  public static $ERROR_EMPTY                  = 'empty';
  /**
   * Error not in given range
   */
  public static $ERROR_RANGE                  = 'range';
  /**
   * Error not matched in regular expression
   */
  public static $ERROR_REGEXP                 = 'regexp';
  /**
   * Error not matched in compare database
   */
  public static $ERROR_COMPAREDB              = 'comparedb';
  /**
   * Error not matched in compare fields
   */
  public static $ERROR_COMPAREFIELD           = 'comparefield';
  /**
   * Error invalid (generic)
   */
  public static $ERROR_INVALID                = 'invalid';
  /**
   * Error invalid type for upload
   */
  public static $ERROR_INVALIDTYPE            = 'invalidtype';
  /**
   * Generic error for upload
   */
  public static $ERROR_UPLOADERROR            = 'uploaderror';

  /**
   * Constructor
   * 
   * @param myMVC_HttpRequest $request  The current request
   * @param myMVC_HttpSession $session  The current session
   */
  public function __construct($request, $session)
  {
    $this->renderEngine = NULL;
    $this->request = $request;
    $this->session = $session;
    $this->checkFunctionAll = $this->checkFunctionPost = $this->checkFunctionGet = $this->checkFunctionAjax = NULL;
  }

  /**
   * Save the current used render engine.
   * It can be used to instanciate Mailview to be used in MessageManager
   * 
   * @param Class $renderEngine         The render engine
   */
  public function saveRenderEngine($renderEngine)
  {
  	$this->renderEngine = $renderEngine;
  }

  /**
   * Return the current used render engine.
   * 
   * @return Class                      The render engine
   */
  protected function getRenderEngine()
  {
    return $this->renderEngine;
  }

  /**
   * Set the functions to be used to validate the user's input.
   * 
   * @param string $type                The type of the request (ALL, POST, GET, AJAX)
   * @param string $func                The method of the current class, to validate the input.
   *                                    It must return a boolean value (true: all valid,
   *                                    false: at least one field not validated)
   */
  protected function setCheckFunction($type, $func)
  {
    if(!method_exists($this, $func))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Check function %s does not exists", $func));
      throw new myMVC_MVCException($msg);
    }
    switch($type)
    {
      case 'ALL':
        $this->checkFunctionAll = $func;
      break;
      case 'POST':
        $this->checkFunctionPost = $func;
      break;
      case 'GET':
        $this->checkFunctionGet = $func;
      break;
      case 'AJAX':
        $this->checkFunctionAjax = $func;
      break;
      default:
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Type %s is invalid. Please use ALL, POST or GET", $type));
        throw new myMVC_MVCException($msg);
      break;
    }
    if($this->checkFunctionAll !== NULL &&
       ($this->checkFunctionPost !== NULL || $this->checkFunctionGet !== NULL || $this->checkFunctionAjax !== NULL))
      myMVC_Logger::doLog(__METHOD__, __LINE__, 'Check type ALL defined and AJAX, POST or GET, too. They will be ignored', ALERT_WARNING);
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                     The token
   */
  abstract public static function getActionToken();

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                     The menu item
   */
  public static function getActionMenuName()
  {
    return '';
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return NULL;
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                    true if the user may call this Action, false otherwise
   */
  abstract public static function hasUserRightForAction();

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'default';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                    true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return false;
  }

  /**
   * Returns whether this action was called from the given view
   * 
   * @param class $views                The view
   * @return boolean                    true if the action was called from the given view, false otherwise
   */
  protected function isComingFromView($view)
  {
    $ref = $this->request->getReferrer();
    return($ref['type'] == 'view' && $ref['name'] == call_user_func(array($view, '::getViewToken')));
  }

  /**
   * Returns if the current action was called from menu or from a view
   * 
   * @return boolean                    true if the action was called from a menu, false if it was called from a view
   */
  protected function isCalledFromMenu()
  {
    return ($this->request->getQueryType() == 'menu');
  }

  /**
   * Validate the data from user.
   * It call the proper validate function, depending on the request method (GET or POST),
   * to allow different checks.
   * WARNING: this function should not be overrided! Please create proper
   * validatePost() or validateGet() functions!
   * 
   * @return boolean                    true if data are valid, false otherwise
   */
  public function validate()
  {
    if($this->checkFunctionAll !== NULL)
      return call_user_method($this->checkFunctionAll, $this);
    if($this->request->getQueryType() == 'ajax' && $this->checkFunctionAjax !== NULL)
      return call_user_method($this->checkFunctionAjax, $this);
    else
    {
      if($this->request->getRequestMethod() == 'POST')
      {
        if($this->checkFunctionPost !== NULL)
          return call_user_method($this->checkFunctionPost, $this);
        else if($this->checkFunctionAll !== NULL)
          return call_user_method($this->checkFunctionAll, $this);
        else
          return true;
      }
      if($this->request->getRequestMethod() == 'GET')
      {
        if($this->checkFunctionGet !== NULL)
          return call_user_method($this->checkFunctionGet, $this);
        else if($this->checkFunctionAll !== NULL)
          return call_user_method($this->checkFunctionAll, $this);
        else
          return true;
      }
    }
    return false;
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse         Response code for the next Action or View
   */
  public function perform()
  {
    return new myMVC_HttpResponse();
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse         Response code for the next Action or View
   *                                    (typically the same view, with error messages)
   */
  public function manageInvalidData()
  {
    return new myMVC_HttpResponse();
  }

  /**
   * Create the error string for the given field.
   * 
   * @param array $definitions          The definition of the fields to validate as
   *                                    associative array ('field' => VALIDATE_TYPE)
   * @param array $names                The names of the fields to be used for displaying errors as
   *                                    associative array ('field' => 'name')
   * @param array $errorMsgs            The error messages for the fields as
   *                                    associative array ('field' => array('error' => 'message'))
   * @param string $field               The checked field
   * @param string $check               The check that field does not passed
   * @param string $externError         Optional extra error (given from extern parsed check function). Default: NULL
   * @return string                     The formatted error string 
   */
  private function createErrorString($definitions, $names, $errorMsgs, $field, $check, $externError = NULL)
  {
    if(isset($errorMsgs[$field][$check]))
      return str_replace(array('{field}', '{checkValue}', '{externError}'),
                         array($names[$field], $definitions[$field][$check], $externError),
                               $errorMsgs[$field][$check]);
    else
      return sprintf(_("%s is not valid"), $names[$field]);
  }

  /**
   * Check if the given value from user's form comply with the given
   * restrictions (min, max, range, and so on).
   * It does NOT check for empty value (this must be done before the
   * first check).
   * 
   * @param array $checks               The required checks as numeric array
   * @param mixed $userInput            User's input
   * @param array $parameters           The parameters to check the field
   * @return string                     The errors for this field (see myMVC_BaseAction::$ERROR_xxx)
   *                                    or false if no errors occoured
   */
  private function checkExtraParameters($checks, $userInput, $parameters)
  {
    $errorsField = '';
    foreach($checks as $check)
    {
      if($check == 'max')
      {
        if(isset($parameters['max']) && $userInput > $parameters['max'])
          return self::$ERROR_MAX;
      }
      elseif($check == 'min')
      {
        if(isset($parameters['min']) && $userInput < $parameters['min'])
          return self::$ERROR_MIN;
      }
      elseif($check == 'maxlen')
      {
        if(isset($parameters['maxlen']) && strlen($userInput) > $parameters['maxlen'])
          return self::$ERROR_MAXLEN;
      }
      elseif($check == 'minlen')
      {
        if(isset($parameters['minlen']) && strlen($userInput) < $parameters['minlen'])
          return self::$ERROR_MINLEN;
      }
      elseif($check == 'regexp')
      {
        if(isset($parameters['regexp']) && !preg_match($parameters['regexp'], $userInput))
          return self::$ERROR_REGEXP;
      }
      elseif($check == 'range')
      {
        if(isset($parameters['range']) && !in_array($userInput, $parameters['range']))
          return self::$ERROR_RANGE;
      }
      elseif($check == 'rangesql')
      {
        if(isset($parameters['rangesql']))
        {
          if(($res = myMVC_GenericSQL::executeQuery($parameters['rangesql'], true)) !== NULL)
          {
            if(!in_array($userInput, $res))
              return self::$ERROR_RANGE;
          }
        }
      }
      elseif($check == 'rangei')
      {
        if(isset($parameters['rangei']))
        {
          $found = false;
          foreach($parameters['rangei'] as $k)
            if(strcasecmp($userInput, $k) == 0)
              $found = true;
          if(!$found)
            return self::$ERROR_RANGE;
        }
      }
      elseif($check == 'rangesqli')
      {
        if(isset($parameters['rangesqli']))
        {
          if(($res = myMVC_GenericSQL::executeQuery($parameters['rangesqli'], true)) !== NULL)
          {
            $found = false;
            foreach($res as $k)
              if(strcasecmp($userInput, $k) == 0)
                $found = true;
            if(!$found)
              return self::$ERROR_RANGE;
          }
        }
      }
      elseif($check == 'comparedb')
      {
        if(isset($parameters['comparedb']))
        {
          if(!is_array($parameters['comparedb']))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'The parameters for comparedb must be an array');
            throw new myMVC_MVCException($msg);
          }
          if(!isset($parameters['comparedb']['query']) || !isset($parameters['comparedb']['compareType']) || !isset($parameters['comparedb']['compareValue']))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Comparedb must have the parameters \'query\', \'compareType\' and \'compareValue\'');
            throw new myMVC_MVCException($msg);
          }
          if(!in_array($parameters['comparedb']['compareType'], array('==', '!=', '<', '>', '<=', '>=')))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, '\'compareType\' must be one of follows: ==, !=, <, >, <=, >=');
            throw new myMVC_MVCException($msg);
          }
          if(($res = myMVC_GenericSQL::executeQuery(str_replace('{checkValue}', myMVC_GenericSQL::quoteSql($userInput), $parameters['comparedb']['query']), true)) === NULL)
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Invalid SQL query');
            throw new myMVC_MVCException($msg);
          }
          if(count($res) != 1)
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'The SQL query must return just one record with one column');
            throw new myMVC_MVCException($msg);
          }
          $match = false;
          switch($parameters['comparedb']['compareType'])
          {
            case '==':
              if($parameters['comparedb']['compareValue'] == $res[0])
                $match = true;
            break;
            case '!=':
              if($parameters['comparedb']['compareValue'] != $res[0])
                $match = true;
            break;
            case '<':
              if($parameters['comparedb']['compareValue'] > $res[0])
                $match = true;
            break;
            case '>':
              if($parameters['comparedb']['compareValue'] < $res[0])
                $match = true;
            break;
            case '>=':
              if($parameters['comparedb']['compareValue'] <= $res[0])
                $match = true;
            break;
            case '<=':
              if($parameters['comparedb']['compareValue'] >= $res[0])
                $match = true;
            break;
          }
          if(!$match)
            return self::$ERROR_COMPAREDB;
        }
      }
      elseif($check == 'comparefield')
      {
        if(isset($parameters['comparefield']))
        {
          if(!is_array($parameters['comparefield']))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'The parameters for comparefield must be an array');
            throw new myMVC_MVCException($msg);
          }
          if(!isset($parameters['comparefield']['field']) || !isset($parameters['comparefield']['compareType']))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Comparefield must have the parameters \'field\' and \'compareType\'');
            throw new myMVC_MVCException($msg);
          }
          if(!in_array($parameters['comparefield']['compareType'], array('==', '!=', '<', '>', '<=', '>=')))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, '\'compareType\' must be one of follows: ==, !=, <, >, <=, >=');
            throw new myMVC_MVCException($msg);
          }
          $fieldValue = $this->request->getParameter($parameters['comparefield']['field']);
          $match = false;
          switch($parameters['comparefield']['compareType'])
          {
            case '==':
              if($userInput == $fieldValue)
                $match = true;
            break;
            case '!=':
              if($userInput != $fieldValue)
                $match = true;
            break;
            case '<':
              if($userInput > $fieldValue)
                $match = true;
            break;
            case '>':
              if($userInput < $fieldValue)
                $match = true;
            break;
            case '>=':
              if($userInput <= $fieldValue)
                $match = true;
            break;
            case '<=':
              if($userInput >= $fieldValue)
                $match = true;
            break;
          }
          if(!$match)
            return self::$ERROR_COMPAREFIELD;
        }
      }
    }
    return false;
  }

  /**
   * Check if the given data (from Request) are valid.
   * The function save in the session (namespace: validate) an associative array with the invalid fields.
   * 
   * @param array $definitions          The definition of the fields to validate as
   *                                    associative array ('field' => VALIDATE_TYPE)
   * @param array $names                The names of the fields to be used for displaying errors as
   *                                    associative array ('field' => 'name')
   * @param array $errorMsgs            The error messages for the fields as
   *                                    associative array ('field' => array('error' => 'message'))
   * @return boolean                    true if ALL fields are valid, false otherwise
   * @throws myMVC_MVCException         On problems
   */
  protected function validateInput($definitions, $names, $errorMsgs)
  {
    $returnVal = true;
    $errors = array();
    $this->session->emptyGivenNamespace('FORMCHECK');
    foreach(array_keys($definitions) as $def)
    {
      $valid = true;
      $userInput = $this->request->getParameter($def);
      $type = $definitions[$def]['type'];
      switch($type)
      {
        case self::$VALIDATE_BOOLEAN:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && $test != 'empty')
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for BOOLEAN. Ignoring!", $test), ALERT_WARNING);
          if(isset($definitions[$def]['empty']) && $definitions[$def]['empty'] === false && $userInput == '')
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_EMPTY);
            break;
          }
          if($userInput != '')
            if(!in_array(strtolower($userInput), array('y', 'n', 't', 'f', '1', '0', 'true', 'false', 'yes', 'no')))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTBOOL);
              break;
            }
        break;
        case self::$VALIDATE_NUMBER:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && !in_array($test, array('empty', 'max', 'min', 'range', 'rangesql', 'comparedb', 'comparefield')))
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for NUMBER. Ignoring!", $test), ALERT_WARNING);
          if(isset($definitions[$def]['empty']) && $definitions[$def]['empty'] === false && $userInput == '')
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_EMPTY);
            break;
          }
          if($userInput != '')
          {
// The locale will be checked!!
            $uInput = $this->getNumber($userInput);
            if(!is_numeric($uInput))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTNUMBER);
              break;
            }
            if(($err = $this->checkExtraParameters(array('max', 'min', 'range', 'rangesql', 'comparedb'), $uInput, $definitions[$def])) !== false)
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
              break;
            }
          }
          if(($err = $this->checkExtraParameters(array('comparefield'), $uInput, $definitions[$def])) !== false)
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
            break;
          }
        break;
        case self::$VALIDATE_DATETIME:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && !in_array($test, array('empty', 'min', 'max', 'format', 'range', 'rangesql', 'comparedb', 'comparefield')))
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for DATETIME. Ignoring!", $test), ALERT_WARNING);
          if(!isset($definitions[$def]['format']))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Mandatory parameter 'format' for the field %s was not given", $def));
            throw new myMVC_MVCException($msg);
          }
          if(isset($definitions[$def]['empty']) && $definitions[$def]['empty'] === false && $userInput == '')
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_EMPTY);
            break;
          }
          if($userInput != '')
          {
            $res = strptime($userInput, $definitions[$def]['format']);
            if($res === false || ($res !== false && $res['unparsed'] != ''))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTDATETIME);
              break;
            }
            else
            {
              if($res['tm_mday'] != '' && $res['tm_mon'] != '' && $res['tm_year'] != '' &&
                 !checkdate($res['tm_mon'] + 1, $res['tm_mday'], $res['tm_year'] + 1900))
              {
                $valid = false;
                $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTDATETIME);
                break;
              }
              if($res['tm_hour'] != '' && $res['tm_min'] != '' && $res['tm_sec'] != '')
              {
                if($res['tm_hour'] < 0 || $res['tm_hour'] > 23 ||
                   $res['tm_min'] < 0 || $res['tm_min'] > 59 ||
                   $res['tm_sec'] < 0 || $res['tm_sec'] > 59)
                {
                  $valid = false;
                  $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTDATETIME);
                  break;
                }
              }
              if(($err = $this->checkExtraParameters(array('range', 'rangesql', 'comparedb'), $userInput, $definitions[$def])) !== false)
              {
                $valid = false;
                $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
                break;
              }
              if(isset($definitions[$def]['max']))
              {
                $uInput = mktime($res['tm_hour'], $res['tm_min'], $res['tm_sec'], $res['tm_mon'] + 1, $res['tm_mday'], $res['tm_year'] + 1900);
                $rCmp = strptime($definitions[$def]['max'], $definitions[$def]['format']);
                $uCmp = mktime($rCmp['tm_hour'], $rCmp['tm_min'], $rCmp['tm_sec'], $rCmp['tm_mon'] + 1, $rCmp['tm_mday'], $rCmp['tm_year'] + 1900);
                if($uInput > $uCmp)
                {
                  $valid = false;
                  $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_MAX);
                  break;
                }
              }
              if(isset($definitions[$def]['min']))
              {
                $uInput = mktime($res['tm_hour'], $res['tm_min'], $res['tm_sec'], $res['tm_mon'] + 1, $res['tm_mday'], $res['tm_year'] + 1900);
                $rCmp = strptime($definitions[$def]['min'], $definitions[$def]['format']);
                $uCmp = mktime($rCmp['tm_hour'], $rCmp['tm_min'], $rCmp['tm_sec'], $rCmp['tm_mon'] + 1, $rCmp['tm_mday'], $rCmp['tm_year'] + 1900);
                if($uInput < $uCmp)
                {
                  $valid = false;
                  $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_MIN);
                  break;
                }
              }
            }
          }
          if(($err = $this->checkExtraParameters(array('comparefield'), $userInput, $definitions[$def])) !== false)
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
            break;
          }
        break;
        case self::$VALIDATE_EMAIL:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && !in_array($test, array('empty', 'range', 'rangesql', 'rangei', 'rangesqli', 'comparedb', 'comparefield')))
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for EMAIL. Ignoring!", $test), ALERT_WARNING);
          if(isset($definitions[$def]['empty']) && $definitions[$def]['empty'] === false && $userInput == '')
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_EMPTY);
            break;
          }
          if($userInput != '')
          {
            if(!myMVC_TypeCheck::checkEMailNotNull($userInput, 'check'))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTEMAIL);
              break;
            }
            if(($err = $this->checkExtraParameters(array('range', 'rangesql', 'rangei', 'rangesqli', 'comparedb'), $userInput, $definitions[$def])) !== false)
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
              break;
            }
          }
          if(($err = $this->checkExtraParameters(array('comparefield'), $userInput, $definitions[$def])) !== false)
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
            break;
          }
        break;
        case self::$VALIDATE_WEB:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && !in_array($test, array('empty', 'justdomain', 'range', 'rangesql', 'rangei', 'rangesqli', 'comparedb', 'comparefield')))
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for WEB. Ignoring!", $test), ALERT_WARNING);
          if(isset($definitions[$def]['empty']) && $definitions[$def]['empty'] === false && $userInput == '')
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_EMPTY);
            break;
          }
          if($userInput != '')
          {
            if(isset($definitions[$def]['justdomain']) && $definitions[$def]['justdomain'] === true)
            {
              if(!myMVC_TypeCheck::checkDomainnameNotNull($userInput, 'check'))
              {
                $valid = false;
                $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTWEB);
                break;
              }
            }
            else
            {
              if(!myMVC_TypeCheck::checkURLNotNull($userInput, 'check'))
              {
                $valid = false;
                $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTWEB);
                break;
              }
            }
            if(($err = $this->checkExtraParameters(array('range', 'rangesql', 'rangei', 'rangesqli', 'comparedb'), $userInput, $definitions[$def])) !== false)
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
              break;
            }
          }
          if(($err = $this->checkExtraParameters(array('comparefield'), $userInput, $definitions[$def])) !== false)
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
            break;
          }
        break;
        case self::$VALIDATE_STRING:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && !in_array($test, array('empty', 'maxlen', 'minlen', 'range', 'rangesql', 'rangei', 'rangesqli', 'regexp', 'comparedb', 'comparefield')))
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for STRING. Ignoring!", $test), ALERT_WARNING);
          if(isset($definitions[$def]['empty']) && $definitions[$def]['empty'] === false && $userInput == '')
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_EMPTY);
            break;
          }
          if($userInput != '')
          {
            if(!is_string($userInput))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_NOTSTRING);
              break;
            }
            if(($err = $this->checkExtraParameters(array('maxlen', 'minlen', 'range', 'rangesql', 'rangei', 'rangesqli', 'regexp', 'comparedb'), $userInput, $definitions[$def])) !== false)
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
              break;
            }
          }
          if(($err = $this->checkExtraParameters(array('comparefield'), $userInput, $definitions[$def])) !== false)
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, $err);
            break;
          }
        break;
        case self::$VALIDATE_EXTERN:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && $test != 'checkFunction')
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for EXTERN. Ignoring!", $test), ALERT_WARNING);
          $errMsg = '';
          if(!isset($definitions[$def]['checkFunction']))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a valid check function. Aborting check", $definitions[$def]['checkFunction']));
            throw new myMVC_MVCException($msg);
            break;
          }
          list($class, $func) = explode('::', $definitions[$def]['checkFunction']);
          if(!class_exists($class))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Class %s is not defined", $class));
            throw new myMVC_MVCException($msg);
          }
          if(!method_exists($class, $func))
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Method %s of class %s is not defined", $func, $class));
            throw new myMVC_MVCException($msg);
          }
          if(!call_user_func_array(array($class, $func), array($this->request, $userInput, &$errMsg)))
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_INVALID, $errMsg);
            break;
          }
        break;
        case self::$VALIDATE_UPLOAD:
          foreach(array_keys($definitions[$def]) as $test)
            if($test != 'type' && !in_array($test, array('empty', 'maxlen', 'minlen', 'allowed_types')))
              myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized test type for UPLOAD. Ignoring!", $test), ALERT_WARNING);
          $file = $this->request->getUpload($def);
          if($file['error'] != UPLOAD_ERR_OK && $file['error'] != UPLOAD_ERR_NO_FILE)
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_UPLOADERROR);
            break;
          }
          if(isset($definitions[$def]['empty']) && $definitions[$def]['empty'] === false && $file['size'] == 0)
          {
            $valid = false;
            $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_EMPTY);
            break;
          }
          if($file['size'] != '')
          {
            if(isset($definitions[$def]['minlen']) && $file['size'] < ($definitions[$def]['minlen'] * 1024))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_MINLEN);
              break;
            }
            if(isset($definitions[$def]['maxlen']) && $file['size'] > ($definitions[$def]['maxlen'] * 1024))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_MAXLEN);
              break;
            }
            if(is_array($definitions[$def]['allowed_types']) && !in_array(strtolower($file['type']), $definitions[$def]['allowed_types']))
            {
              $valid = false;
              $errors[$def] .= $this->createErrorString($definitions, $names, $errorMsgs, $def, self::$ERROR_INVALIDTYPE);
              break;
            }
          }
        break;
        default:
          myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s is not a recognized check type. Ignoring!", $type), ALERT_WARNING);
        break;
      }
      $this->session->setInNamespace('FORMCHECK', 'CHECKED-'.$def, true);
      if($valid)
      {
        $this->request->setParameterValid($def);
        $this->session->setInNamespace('FORMCHECK', 'VALID-'.$def, true);
      }
      else
      {
        $this->session->setInNamespace('FORMCHECK', 'VALID-'.$def, false);
        $returnVal = false;
      }
    }
    $this->session->setInNamespace('FORMCHECK', 'ERRORS', $errors);
    return $returnVal;
  }

  /**
   * Convert a number to the internal PHP-Format, using current user's locale
   * 
   * @param string $value               The number to be converted
   * @return number                     The converted number
   */
  protected function getNumber($value)
  {
    $localeInfo = localeconv();
    $val = str_replace($localeInfo['thousands_sep'], '', $value);
    return str_replace($localeInfo['decimal_point'], '.', $val);
  }
}
?>
