<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Base class for the Views.
 * All user's view <b>must</b> derive this class
 */
abstract class myMVC_BaseView extends myMVC_BasePage
{
  protected $engine;
  protected $session;
  protected $templateFile;
  protected $variables;
  protected $objects;

  /**
   * Constructor.
   * 
   * @param class $engine               The engine as subclass of myMVC_BaseEngine
   * @param myMVC_HttpSession $session  The current session
   */
  public function __construct($engine, $session)
  {
    $this->engine = $engine;
    $this->session = $session;
    $this->templateFile = NULL;
    $this->variables = array();
    $this->objects = array();
  }

  /**
   * Save a variable to be used in the template
   * 
   * @param string $key                 The name of the template variable
   * @param string $value               The value
   */
  public function setVariable($key, $value)
  {
    $this->variables[$key] = $value;
  }

  /**
   * Save an object (reference) to be used in the template
   * 
   * @param string $key                 The name of the template object
   * @param mixed $value                The object
   */
  public function setObject($key, $obj)
  {
    $this->objects[$key] = $obj;
  }

  /**
   * Returns the template file for the current view
   * 
   * @throws myMVC_MVCException         On problems
   */
  public function getTemplateFile()
  {
    if($this->templateFile === NULL)
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'No template file given');
      throw new myMVC_MVCException($msg);
    }
    return $this->templateFile;
  }

  /**
   * Use $file as template
   * 
   * @param string $file                The template file
   */
  protected function setTemplate($file)
  {
    if(!is_file($this->engine->getTemplateDirectory().'/'.$file))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("%s is not regular file", $file));
      throw new myMVC_MVCException($msg);
    }
    if(!is_readable($this->engine->getTemplateDirectory().'/'.$file))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("File %s is not readable", $file));
      throw new myMVC_MVCException($msg);
    }
    $this->templateFile = $file;
  }

  /**
   * Returns the token used to identify the current View
   * 
   * @return string                     The token
   */
  abstract public static function getViewToken();

  /**
   * Prepare to display the page.
   * Send all variables to the engine
   */
  private function prepare()
  {
    global $config;

    foreach(array_keys($this->variables) as $key)
      $this->engine->assignVariable($key, $this->variables[$key]);
    foreach(array_keys($this->objects) as $key)
      $this->engine->assignObject($key, $this->objects[$key]);

    $this->engine->assignObject('session', $this->session);
    $this->engine->assignVariable('dateFormat', _($config['PHP_DATEFORMAT']));
    $this->engine->assignVariable('timeFormat', _($config['PHP_TIMEFORMAT']));
    $this->engine->assignVariable('dateFormatTime', _($config['PHP_DATETIMEFORMAT']));
    $this->engine->assignObject('menuActions', myMVC_Controller::getMenuActions());

    $this->engine->assignVariable('myMVCCurrentVersion', MYMVC_CURRENTVERSION);

    $this->engine->assignVariable('argSep', ini_get('arg_separator.output'));

    $this->engine->registerFunction('createURL', array('myMVC_BaseView', 'createURL'));
    $this->engine->registerFunction('createURLPrevious', array('myMVC_BaseView', 'createURLPrevious'));
    $this->engine->registerFunction('getActionCSS', array('myMVC_BaseView', 'getActionCSS'));
    $this->engine->registerFunction('menuItem', array('myMVC_BaseView', 'getActionMenuItem'));
    $this->engine->registerFunction('callActionMethod', array('myMVC_BaseView', 'callActionStaticFunction'));
    $this->engine->registerFunction('errorMessage', array('myMVC_BaseView', 'errorMessage'));
    $this->engine->registerFunction('errorMessages', array('myMVC_BaseView', 'errorMessagesAsString'));
    $this->engine->registerFunction('errorMessagesArray', array('myMVC_BaseView', 'errorMessagesAsArray'));
    $this->engine->registerFunction('isFieldValid', array('myMVC_BaseView', 'isFieldValid'));
    $this->engine->registerFunction('isCalledFromMenu', array('myMVC_BaseView', 'isCalledFromMenu'));
    $this->engine->registerModifier('formatNumber', array('myMVC_BaseView', 'formatNumber'));
  }

  /**
   * Set all form fields of the previous form as variable to be displayed by the engine
   */
  protected function setAllSubmitFields()
  {
    $fields = $this->session->getFromNamespace('MVCSAVEDFIELDS', 'allFields');
    if(is_array($fields))
      foreach(array_keys($fields) as $field)
        $this->setVariable($field, $fields[$field]);
  }

  /**
   * Set all the invalid form fields of the previous form as variable to be displayed by the engine
   */
  protected function setAllInvalidFields()
  {
    $fields = $this->session->getFromNamespace('MVCSAVEDFIELDS', 'fieldsInvalid');
    if(is_array($fields))
    {
      foreach(array_keys($fields) as $field)
      {
        if($this->variables[$field] === NULL)
          $this->setVariable($field, $fields[$field]);
      }
    }
  }

  /**
   * Set all the valid form fields of the previous form as variable to be displayed by the engine
   */
  protected function setAllValidFields()
  {
    $fields = $this->session->getFromNamespace('MVCSAVEDFIELDS', 'fieldsValid');
    if(is_array($fields))
      foreach(array_keys($fields) as $field)
        $this->setVariable($field, $fields[$field]);
  }

  /**
   * Prepare to render the View
   * 
   * @return boolean                    true if the page can be display, false otherwise
   */
  public function preRender()
  {
    return true;
  }

  /**
   * Render the page and send it to
   * the browser
   */
  public function render()
  {
    $this->prepare();
    $this->engine->display($this);
  }

  /**
   * Render the page and return it as string
   * 
   * @return string                     The page as string
   */
  public function fetch()
  {
    $this->prepare();
    return $this->engine->fetch($this);
  }

  /**
   * Template-function to create the URL of the previous used action
   * 
   * @param array $param                Associative array with the parameter.
   *                                    If "var" is specified, the URL will be saved in this variable (using the engine)
   *                                    If "menuitem" is specified and his value is true, the URL will be created
   *                                    as menuitem (see myMVC_BaseAction::isCalledFromMenu())
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return string                     The created URL
   */
  public static function createURLPrevious($params, &$engine)
  {
    global $config;

    $session = myMVC_HttpSession::getInstance();
    $className = $session->getFromNamespace('controller', 'previousAction');
    $sessId = '';
    if($config['SESSION_USECOOKIES'] == 0)
      $sessId = ini_get('arg_separator.output').$session->getSessionId();
    if($params['menuitem'] === true)
    {
      $menuActions = myMVC_Controller::getMenuActions();
      if(in_array($params['class'], $menuActions))
        $cmd = 'menu';
      else
      {
        myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Class %s is not registered as menu class. Ignoring menu request", $params['class']), ALERT_WARNING);
        $cmd = 'cmd';
      }
    }
    else
      $cmd = 'cmd';
    $parameters = '';
    if(isset($params['params']))
    {
      if(!is_array($params['params']))
        $parameters = ini_get('arg_separator.output').$params['params'];
      else
        foreach($params['params'] as $p)
          $parameters .= ini_get('arg_separator.output').$p;
    }
    if($params['var'] !== NULL)
      $engine->assign($params['var'], sprintf("?%s=%s%s%s", $cmd, $className, $parameters, $sessId));
    else
      return sprintf("?%s=%s%s%s", $cmd, $className, $parameters, $sessId);
  }

  /**
   * Template-function to create an URL
   * 
   * @param array $param                Associative array with the parameter.
   *                                    Needed is "class" with the class name
   *                                    If "var" is specified, the URL will be saved in this variable (using the engine)
   *                                    If "menuitem" is specified and his value is true, the URL will be created
   *                                    as menuitem (see myMVC_BaseAction::isCalledFromMenu())
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return string                     The created URL
   */
  public static function createURL($params, &$engine)
  {
    global $config;

    if($params['class'] !== NULL)
    {
      $session = myMVC_HttpSession::getInstance();
      $sessId = '';
      if($config['SESSION_USECOOKIES'] == 0)
        $sessId = ini_get('arg_separator.output').$session->getSessionId();
      if(myMVC_Controller::hasUserRightForAction($params['class']))
      {
        $className = call_user_func($params['class'].'::getActionToken');
        if($params['menuitem'] === true)
        {
          $menuActions = myMVC_Controller::getMenuActions();
          if(in_array($params['class'], $menuActions))
            $cmd = 'menu';
          else
          {
            myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Class %s is not registered as menu class. Ignoring menu request", $params['class']), ALERT_WARNING);
            $cmd = 'cmd';
          }
        }
        elseif($params['ajax'] === true)
          $cmd = 'ajax';
        else
          $cmd = 'cmd';
        $parameters = '';
        if(isset($params['params']))
        {
          if(!is_array($params['params']))
            $parameters = ini_get('arg_separator.output').$params['params'];
          else
            foreach($params['params'] as $p)
              $parameters .= ini_get('arg_separator.output').$p;
        }
        if($params['var'] !== NULL)
          $engine->assign($params['var'], sprintf("?%s=%s%s%s", $cmd, $className, $parameters, $sessId));
        else
          return sprintf("?%s=%s%s%s", $cmd, $className, $parameters, $sessId);
      }
      else
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
                sprintf("Action class %s does not exists", $params['class']));
        throw new myMVC_MVCException($msg);
      }
    }
  }

  /**
   * Template-function to return the CSS of the given action
   * 
   * @param array $param                Associative array with the parameter.
   *                                    Needed is "class" with the class name
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return string                     The CSS class of the given action
   */
  public static function getActionCSS($params, &$engine)
  {
    if($params['class'] !== NULL)
    {
      $session = myMVC_HttpSession::getInstance();
      if(myMVC_Controller::hasUserRightForAction($params['class']))
      {
        if($params['class'] !== NULL)
        {
          if($params['var'] !== NULL)
            $engine->assign($params['var'], call_user_func($params['class'] .'::getCSSClass'));
          else
            return call_user_func($params['class'] .'::getCSSClass');
        }
      }
      else
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
                sprintf("Action class %s does not exists", $params['class']));
        throw new myMVC_MVCException($msg);
      }
    }
  }

  /**
   * Template-function to create menu item
   * 
   * @param array $param                Associative array with the parameter.
   *                                    Needed is "class" with the class name
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return string                     The action name
   */
  public static function getActionMenuItem($params, &$smarty)
  {
    if($params['class'] !== NULL)
    {
      $session = myMVC_HttpSession::getInstance();
      if(myMVC_Controller::hasUserRightForAction($params['class']))
      {
        if($params['class'] !== NULL)
          return call_user_func($params['class'] .'::getActionMenuName');
      }
      else
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
                sprintf("Action class %s does not exists", $params['class']));
        throw new myMVC_MVCException($msg);
      }
    }
  }

  /**
   * Template-function to call an arbitrary static function of a given Action class
   * 
   * @param array $param                Associative array with the parameter.
   *                                    Needed are "class" with the class name and
   *                                    "method" as the <b>static</b> method
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return string                     The action name
   */
  public static function callActionStaticFunction($params, &$smarty)
  {
    if($params['class'] !== NULL)
    {
      $session = myMVC_HttpSession::getInstance();
      if(myMVC_Controller::hasUserRightForAction($params['class']))
      {
        if($params['class'] !== NULL)
          return call_user_func($params['class'].'::'.$params['method']);
      }
      else
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
                sprintf("Action class %s does not exists", $params['class']));
        throw new myMVC_MVCException($msg);
      }
    }
  }

  /**
   * Template-function to return the error message of the given form field
   * 
   * @param array $param                Associative array with the parameter.
   *                                    Needed is "field" with the field name
   *                                    If "var" is specified, the URL will be saved in this variable (using the engine)
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return string                     The error message for the given field
   */
  public static function errorMessage($params, &$engine)
  {
    global $config;

    if($params['field'] !== NULL)
    {
      $session = myMVC_HttpSession::getInstance();
      $errors = $session->getFromNamespace('FORMCHECK', 'ERRORS');

      if($params['var'] !== NULL)
        $engine->assign($params['var'], $errors[$params['field']]);
      else
        return $errors[$params['field']];
    }
  }

  /**
   * Template-function to return all error messages as string
   * 
   * @param array $param                Associative array with the parameter.
   *                                    Needed is "format" with a printf-style to describe every line.
   *                                    If "var" is specified, the URL will be saved in this variable (using the engine)
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return string                     All error messages
   */
  public static function errorMessagesAsString($params, &$engine)
  {
    global $config;

    if($params['format'] !== NULL)
    {
      $session = myMVC_HttpSession::getInstance();
      $errorArray = $session->getFromNamespace('FORMCHECK', 'ERRORS');
      $errors = '';

      if(is_array($errorArray))
        foreach(array_keys($errorArray) as $err)
          $errors .= sprintf($params['format']."\n", $errorArray[$err]);
      if($params['var'] !== NULL)
        $engine->assign($params['var'], $errors);
      else
        return $errors;
    }
  }

  /**
   * Template-function to return all error messages as numeric array
   * of associative arrays ('field' => <field name>, 'message' => <error message>)
   * 
   * @param array $param                Associative array with the parameter.
   *                                    If "var" is specified, the URL will be saved in this variable (using the engine)
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return array                      All error messages as numeric array of associative arrays
   */
  public static function errorMessagesAsArray($params, &$engine)
  {
    global $config;

    $session = myMVC_HttpSession::getInstance();
    $errorArray = $session->getFromNamespace('FORMCHECK', 'ERRORS');
    $errors = array();

    if(is_array($errorArray))
      foreach(array_keys($errorArray) as $err)
        array_push($errors, array('field' => $err, 'message' => $errorArray[$err]));
    if($params['var'] !== NULL)
      $engine->assign($params['var'], $errors);
    else
      return $errors;
  }

  /**
   * Template-function to return whether the form field passed the validation
   * 
   * @param array $param                Associative array with the parameter.
   *                                    Needed is "field" with the field name
   *                                    If "var" is specified, the URL will be saved in this variable (using the engine)
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return bool                       true if the field passed the validation, false otherwise
   */
  public static function isFieldValid($params, &$engine)
  {
    global $config;

    if($params['field'] !== NULL)
    {
      $session = myMVC_HttpSession::getInstance();
      if($session->getFromNamespace('FORMCHECK', 'CHECKED-'.$params['field']))
        $valid = $session->getFromNamespace('FORMCHECK', 'VALID-'.$params['field']);
      else
        $valid = true;

      if($params['var'] !== NULL)
        $engine->assign($params['var'], $valid);
      else
        return $valid;
    }
  }

  /**
   * Template-function to return whether the action, who called this view, was called from
   * a menu or directly from another action/view
   * 
   * @param array $param                Associative array with the parameter.
   *                                    If "var" is specified, the URL will be saved in this variable (using the engine)
   * @param $engine                     The engine (typically myMVC_SmartyEngine)
   * @return bool                       true if the field passed the validation, false otherwise
   */
  public static function isCalledFromMenu($params, &$engine)
  {
    global $config;

    $session = myMVC_HttpSession::getInstance();
    $class = myMVC_Controller::getCurrentAction();
    $isMenu = $session->getFromNamespace('MVCRESPONSE-'.$class, 'menu');
    if($params['var'] !== NULL)
      $engine->assign($params['var'], $isMenu);
    else
        return $isMenu;
  }

  /**
   * Format a number with the actual locale information
   * Example:
   * Locale de_DE: 1.234,56
   * Locale en_US: 1,234.56
   * Locale it_IT: 1234,56
   * 
   * @param $number                     The number to be formatted
   * @param $decimals                   How many decimals
   * @return string                     The formatted number
   */
  public static function formatNumber($number, $decimals)
  {
    $locale = localeconv();
    return number_format($number, $decimals, $locale['decimal_point'], $locale['thousands_sep']);
  }
}
?>
