<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Generic database connector.
 * It should <b>not</b> be used directly!
 * Classes, derived from this, will be called from myMVC_BaseDB and derived
 * to manage data in the database.
 */
class myMVC_DatabaseConnector extends myMVC_TypeCheck
{
  protected $server;
  protected $login;
  protected $password;
  protected $database;
  protected $engine;

  /**
   * Constructor. Initializes the database connection
   * WARNING: This function will only called from myMVC_BaseDB
   * constructor, using the proper derived class
   * (see: myMVC_DatabaseConnectorMysql and myMVC_DatabaseConnectorPgsql)
   */
  protected function __construct()
  {
    global $config;

    $this->server     = $config['DATABASE']['SERVER'];
    $this->login      = $config['DATABASE']['USER'];
    $this->password   = $config['DATABASE']['PASSWORD'];  
    $this->database   = $config['DATABASE']['DATABASE'];  
    $this->engine     = $config['DATABASE']['ENGINE'];  
  }

  /**
   * Wrapper for the errno-function of the selected database engine
   * 
   * @return int                the error number
   */
  protected function errno()
  {
    return 0;
  }

  /**
   * Wrapper for the error-function of the selected database engine
   * 
   * @return string             the error string
   */
  protected function error()
  {
    return '';
  }

  /**
   * Connects to the database.
   * This will just be done, if no connection exists yet.
   *  
   * @return resource                 a connection resource
   * @throws myMVC_DatabaseException  if no connection is possible
   */ 
  protected function getDatabaseConnection()
  {
    $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Database engine not specified');
    throw new myMVC_DatabaseException($msg);
  }

  /**
   * Quotes a string to use in a SQL-Query
   * 
   * @param  string $value      The string with the Query
   * @return string             A secure SQL-String (quoted)
   */
  protected function quoteSqlString($value)
  {
    return $this->quoteSqlStringForLike($value, '', '');
  }

  /**
   * Quotes a string to use in a SQL-Query (with LIKE)
   * 
   * @param  string $value      The string with the Query
   * @param  string $prefix     (normally %) added BEFORE the string
   * @param  string $suffix     (normally %) added AFTER the string
   * @return string             A secure SQL-String (quoted)
   */
  protected function quoteSqlStringForLike($value, $prefix = '%', $suffix = '%')
  {
    return $value;
  }

  /**
   * Execute an INSERT, UPDATE or DELETE query
   * 
   * @param  string $query      The SQL-Query
   * @return boolean            true on success, false on failure, last inserted ID on successfully INSERT
   */
  protected function execute($query)
  {
    return false;
  }

  /**
   * Executes a SELECT-Query
   * 
   * @param  string $query      The SQL-Query
   * @return resource           a resource to get data on success, false otherwise
   */
  protected function executeSelect($query)
  {
    return false;
  }

  /**
   * Returns a record as associative array, or NULL if no record can be returned.
   * 
   * @param resource $result    The resource
   * @return array              The record
   */
  protected function getValueArrayFromResult($result)
  {
    return NULL;
  }

  /**
   * Returns a record as numeric array, or NULL if no record can be returned.
   * 
   * @param resource $result    The resource
   * @return array              The record
   */
  protected function getValuesFromResult($result)
  {
    return NULL;
  }

  /**
   * Returns the number of row of the runned query.
   * 
   * @param resource $result    The resource
   * @return int                The row's number
   */
  protected function getRowsNum($result)
  {
    return 0;
  }

  /**
   * Free the allocated memory for the SQL query.
   * 
   * @param resource $result    The resource
   * @return boolean            true on success, false on problem
   */
  protected function freeResult($result)
  {
    return false;
  }
}
?>
