<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Manager for the request
 */
class myMVC_HttpRequest
{
  /**
   * Possible query types
   */
  private static $queryTypes        = array('cmd', 'ajax', 'menu', 'view');

  /**
   * Unknown type. It will <b>not</b> be used. Just for initialize...
   */
  public static $TYPE_UNKNOWN       = 'unknown';
  /**
   * Action type (called from link)
   */
  public static $TYPE_ACTION        = 'cmd';
  /**
   * Action type (called from AJAX)
   */
  public static $TYPE_AJAX          = 'ajax';
  /**
   * Action type (called from menu)
   */
  public static $TYPE_MENU          = 'menu';
  /**
   * View type
   */
  public static $TYPE_VIEW          = 'view';

  private $request;
  private $upload;
  private $checkedFields;
  private $checkedUploads;
  private $server;
  private $cookie;
  private static $instance;

  /**
   * Constructor.
   * Get from PHP engine the request.
   * WARNING: This class has to be used as Singleton!
   * Example:
   * 
   * <code>
   * $request = myMVC_HttpRequest::getInstance();
   * </code>
   */
  private function __construct()
  {
    $this->upload = $_FILES;
    $this->request = $_REQUEST;
    $this->server = $_SERVER;
    $this->cookie = $_COOKIE;
    $this->checkedFields = $this->checkedUploads = array();
  }

  /**
   * Returns the instance of this class.
   * 
   * @return myMVC_HttpRequest      An instance of the request class
   */
  public static function getInstance()
  { 
    if(!isset(self::$instance))
    { 
      $class = __CLASS__; 
      self::$instance = new $class(); 
    } 
    return self::$instance; 
  }

  /**
   * Returns the request method (GET/POST)
   * 
   * @return string                 The request method
   */
  public function getRequestMethod()
  {
    return $this->server['REQUEST_METHOD'];
  }

  /**
   * Returns HTTP query string
   * 
   * @return string                 The HTTP query string
   */
  public function getQueryString()
  {
    return $this->server['QUERY_STRING'];
  }

  /**
   * Returns the referrer of the current page
   * 
   * @return array                  The referrer as associative array (type = cmd|view, name = <Action|View name>)
   *                                or false, if no referrer can be found
   */
  public function getReferrer()
  {
    $regexp = '/^[https\:]+\/\/[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}[\/]?[A-z0-9\w\-\.]*[\?]?([a-z]+)[\=]?([a-zA-Z]+).*/';
    if(preg_match($regexp, $this->server['HTTP_REFERER'], $matches, PREG_OFFSET_CAPTURE) != 0)
      return array('type' => $matches[1][0], 'name' => $matches[2][0]);
    return false;
  }

  /**
   * Returns the query type (Action or View)
   * 
   * @return string                 The query type [cmd|menu|view] or false if no known query was found
   */
  public function getQueryType()
  {
    foreach(self::$queryTypes as $type)
      if(isset($this->request[$type]))
        return $type;
    return false;
  }

  /**
   * Returns all valid query types
   * 
   * @param string $postfix         Postfix to add to all query types
   * @return array                  An array with all valid query types
   */
  public function getQueryTypes($postfix = '')
  {
    $ret = array();
    foreach(self::$queryTypes as $type)
      array_push($ret, $type.$postfix);
    return $ret;
  }

  /**
   * Returns whether the current query is empty (then it must be forwarded to a default)
   * 
   * @return boolean                true if the query is empty, false otherwise
   */
  public function isEmptyRequest()
  {
    foreach(self::$queryTypes as $type)
      if(isset($this->request[$type]))
        return false;
    return true;
  }

  /**
   * Returns the current action or false if no action was called
   * 
   * @return string                 The called action or false if no action was called
   */
  public function getAction()
  {
    if(isset($this->request[self::$TYPE_ACTION]))
      return $this->request[self::$TYPE_ACTION];
    if(isset($this->request[self::$TYPE_AJAX]))
      return $this->request[self::$TYPE_AJAX];
    if(isset($this->request[self::$TYPE_MENU]))
      return $this->request[self::$TYPE_MENU];
    return false;
  }

  /**
   * Returns if the current action was called as AJAX
   * 
   * @return boolean                    true if the current Action was called as AJAX, false otherwise
   */
  public function isAJAX()
  {
    if(isset($this->request[self::$TYPE_AJAX]))
      return true;
    return false;
  }

  /**
   * Returns the current view or false if no action was called
   * 
   * @return string                 The called view or false if no action was called
   */
  public function getView()
  {
    if(isset($this->request[self::$TYPE_VIEW]))
      return $this->request[self::$TYPE_VIEW];
    return false;
  }

  /**
   * Set the given parameter as valid (passed the validation)
   * 
   * @param string $parameter       The name of the valid parameter
   */
  public function setParameterValid($parameter)
  {
    if(isset($this->upload[$parameter]))
      $this->checkedUploads[$parameter] = isset($this->upload[$parameter]) ? $this->upload[$parameter] : NULL;
    else
      $this->checkedFields[$parameter] = isset($this->request[$parameter]) ? $this->request[$parameter] : NULL;
  }

  /**
   * Returns an associative array with all given parameters and their
   * value (format: 'parameter' => 'value')
   * 
   * @param bool $checked           If true (default), then just checked parameter will be returned
   */
  public function getAllParameters($checked = true)
  {
    $ret = array();
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    if(is_subclass_of($caller, 'myMVC_BaseAction'))
      $f = $this->checkedFields;
    else
    {
      if($checked)
        $f = $this->checkedFields;
      else
        $f = $this->request;
    }
    foreach(array_keys($f) as $k)
      if($k != self::$TYPE_ACTION && $k != self::$TYPE_AJAX && $k != self::$TYPE_MENU && $k != self::$TYPE_VIEW && $k != session_name())
        $ret[$k] = $f[$k];
    return $ret;
  }

  /**
   * Returns an associative array with all parameters, that passed the validation,
   * and their value (format: 'parameter' => 'value')
   */
  public function getInvalidParameters()
  {
    $ret = array();
    foreach(array_keys($this->request) as $k)
      if($k != self::$TYPE_ACTION && $k != self::$TYPE_AJAX && $k != self::$TYPE_MENU && $k != self::$TYPE_VIEW && $k != session_name() && !in_array($k, array_keys($this->checkedFields)))
        $ret[$k] = $this->request[$k];
    return $ret;
  }

  /**
   * Returns whether the given <b>unvalidated</b> parameter exists
   * 
   * @param string $key             The name of the parameter
   * @return boolean                true if it exists, false otherwise
   */
  public function hasSubmitParameter($key)
  {
    if(isset($this->request[$key]))
      return true;
    else
      return false;
  }

  /**
   * Returns whether the an <b>unvalidated</b> parameter starting with $key exists
   * 
   * @param string $key             The name of the parameter
   * @return boolean                true if it exists, false otherwise
   */
  public function hasSubmitParameterLike($key)
  {
    foreach(array_keys($this->request) as $k)
      if(strncmp($k, $key, strlen($key)) == 0)
        return true;
    return false;
  }

  /**
   * Returns the <b>unvalidated</b> value for the given parameter.
   * Useful if a form has more submit buttons and you want to check
   * which button was used
   * 
   * @param string $key             The name of the parameter
   * @return string                 The value of the parameters or false if it doesn't exists
   */
  public function getSubmitParameter($key)
  {
    if(isset($this->request[$key]))
      return $this->request[$key];
    else
      return false;
  }

  /**
   * Returns the <b>unvalidated</b> value for the parameter starting with $key.
   * Useful if a form has more submit buttons and you want to check
   * which button was used
   * 
   * @param string $key             The name of the parameter
   * @return string                 The value of the parameters or false if it doesn't exists
   */
  public function getSubmitParameterLike($key)
  {
    foreach(array_keys($this->request) as $k)
      if(strncmp($k, $key, strlen($key)) == 0)
        return $this->request[$k];
    return false;
  }

  /**
   * Returns the <b>name</b> of the parameter starting with $key.
   * Useful if a form has more submit buttons and you want to check
   * which button was used
   * 
   * @param string $key             The name of the parameter
   * @return string                 The name of the parameters or false if it doesn't exists
   */
  public function getSubmitParameterNameLike($key)
  {
    foreach(array_keys($this->request) as $k)
      if(strncmp($k, $key, strlen($key)) == 0)
        return $k;
    return false;
  }

  /**
   * Returns whether the given parameter exists
   * 
   * @param string $key             The name of the parameter
   * @return boolean                true if it exists, false otherwise
   */
  public function hasParameter($key)
  {
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    if(is_subclass_of($caller, 'myMVC_BaseAction'))
      $f = $this->checkedFields;
    else
      $f = $this->request;
    if(isset($f[$key]))
      return true;
    return false;
  }

  /**
   * Returns whether the a parameter starting with $key exists
   * 
   * @param string $key             The name of the parameter
   * @return boolean                true if it exists, false otherwise
   */
  public function hasParameterLike($key)
  {
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    if(is_subclass_of($caller, 'myMVC_BaseAction'))
      $f = $this->checkedFields;
    else
      $f = $this->request;
    foreach(array_keys($f) as $k)
      if(strncmp($k, $key, strlen($key)) == 0)
        return true;
    return false;
  }

  /**
   * Returns the value of the given parameter
   * 
   * @param string $key             The name of the parameter
   * @return string                 The value of the parameters or false if it doesn't exists
   */
  public function getParameter($key)
  {
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    if(is_subclass_of($caller, 'myMVC_BaseAction'))
      $f = $this->checkedFields;
    else
      $f = $this->request;
    if(isset($f[$key]))
      return $f[$key];
    return false;
  }

  /**
   * Returns the value of a parameter starting with $key
   * 
   * @param string $key             The name of the parameter
   * @return string                 The value of the parameters or false if it doesn't exists
   */
  public function getParameterLike($key)
  {
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    if(is_subclass_of($caller, 'myMVC_BaseAction'))
      $f = $this->checkedFields;
    else
      $f = $this->request;
    foreach(array_keys($f) as $k)
      if(strncmp($k, $key, strlen($key)) == 0)
        return $f[$k];
    return false;
  }

  /**
   * Set a value for a parameter (useful to call other pages)
   * 
   * @param string $key             The name of the parameter
   * @param string $value           The value
   */
  public function setParameter($key, $value)
  {
    $this->request[$key] = $value;
  }


  /**
   * Returns the <b>unvalidated</b> value for the given upload.
   * Useful if a form has more submit buttons and you want to check
   * which button was used
   * 
   * @param string $key             The name of the parameter
   * @return array                  The value of the parameters or false if it doesn't exists
   */
  public function getSubmitUpload($key)
  {
    if(isset($this->upload[$key]))
      return $this->upload[$key];
    else
      return false;
  }
  
  /**
   * Returns whether the given upload exists and a file
   * was uploaded
   *
   * @param string $key             The name of the parameter
   * @return boolean                true if it exists, false otherwise
   */
  public function hasUpload($key)
  {
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    if(is_subclass_of($caller, 'myMVC_BaseAction'))
      $f = $this->checkedUploads;
    else
      $f = $this->upload;
    if(isset($f[$key]) && $f[$key]['error'] == UPLOAD_ERR_OK)
      return true;
    return false;
  }

  /**
   * Return the upload file, if given
   * 
   * @param string $key             The name of the parameter
   * @return array                  Associative array with the file data ot false if no file uploaded
   *                                Keys: name, type, size, tmp_name, error
   */
  public function getUpload($key)
  {
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    if(is_subclass_of($caller, 'myMVC_BaseAction'))
      $f = $this->checkedUploads;
    else
      $f = $this->upload;
    if(isset($f[$key]) && $f[$key]['error'] == UPLOAD_ERR_OK)
      return $f[$key];
    return false;
  }
}
?>
